﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Example07_Clockgroup1
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        // コンストラクター
        public MainWindow()
        {
            InitializeComponent();
            InitializeWindow();
        }

        private const int NUM = 4;        // Rectangleの数
        private const int TIME = 4;       // アニメーションの所要時間
        private ClockGroup _cgroup;

        // [Avi]ボタンクリックイベント
        private void btnAvi_Click(object sender, RoutedEventArgs e)
        {
            var filepath = GetAviFilePath();
            if (0 < filepath.Length)
            {
                SaveAsAvi(filepath);
                MessageBox.Show("OK");
            }
        }

        // [開始]ボタンクリックイベント
        private void btnStart_Click(object sender, RoutedEventArgs e)
        {
            _cgroup.Controller.Begin();
        }

        // [停止]ボタンクリックイベント
        private void btnStop_Click(object sender, RoutedEventArgs e)
        {
            _cgroup.Controller.Stop();
        }


        // Aviファイル保存パス取得
        private string GetAviFilePath()
        {
            var dlg = new Microsoft.Win32.SaveFileDialog();
            dlg.Filter = "aviファイル|*.avi;*.avi";
            var path = String.Empty;

            if (dlg.ShowDialog() == true)
            {
                path = dlg.FileName;
            }

            return path;
        }

        // Rectangleの配置とRectangleへのアニメーションの適用
        private void InitializeWindow()
        {
            var pt = new ParallelTimeline();
            Rectangle[] rect = new Rectangle[NUM];
            Color[] color = { Colors.Red, Colors.Blue, Colors.Magenta, Colors.Green };

            for (int i = 0; i < NUM; i++)
            {
                // Rectangleの生成
                rect[i] = new Rectangle();
                rect[i].Width = 60;
                rect[i].Height = 60;
                Canvas.SetLeft(rect[i], 10);
                Canvas.SetTop(rect[i], i * 65 + 10);
                rect[i].Fill = new SolidColorBrush(color[i]);
                cvs1.Children.Add(rect[i]);

                // アニメーション生成
                var da = new DoubleAnimation(10, 460, new Duration(TimeSpan.FromSeconds(TIME)));
                da.BeginTime = TimeSpan.FromSeconds(i);
                da.AutoReverse = true;
                //da.RepeatBehavior = new RepeatBehavior(2);
                pt.Children.Add(da);
            }

            _cgroup = pt.CreateClock();
            for (int i2 = 0; i2 < NUM; i2++)
            {
                rect[i2].ApplyAnimationClock(Canvas.LeftProperty, 
                                            (AnimationClock)_cgroup.Children[i2]);
            }
            _cgroup.Controller.Stop();
        }

        // Aviファイル作成
        private void SaveAsAvi(string filepath)
        {
            var fps = 48;                              // フレームレート
            var totalFrames = fps * (TIME * 2 + 4);    // 全フレーム数
            var secs = Enumerable.Range(0, totalFrames).Select(t => (((double)t) / fps));
            var aviManager = new AviFile.AviManager(filepath, false);
            AviFile.VideoStream aviStream = null;
            var tempFile = "frame.png";

            foreach (var sec in secs)
            {
                _cgroup.Controller.SeekAlignedToLastTick(TimeSpan.FromSeconds(sec),
                                                         TimeSeekOrigin.BeginTime);
                cvs1.UpdateLayout();
                cvs1.SaveImage(tempFile);
                var bmp = new System.Drawing.Bitmap(tempFile);
                if (aviStream == null)
                    aviStream = aviManager.AddVideoStream(true, fps, bmp);
                else
                    aviStream.AddFrame(bmp);
                bmp.Dispose();
            }
            aviManager.Close();
            _cgroup.Controller.Stop();
            System.IO.File.Delete(tempFile);
        }

    }

    public static class CanvasExtension
    {
        /// <summary> [SaveImage]：Canvasの画像をファイルに保存するCanvasクラスの拡張メソッド 
        /// 出典：http://qiita.com/hugo-sb/items/894914f6bbe224a45d49
        /// </summary>
        /// <param name="canvas"></param>
        /// <param name="path">保存する画像ファイル名</param>
        /// <param name="encoder">BitmapEncoderが指定されなかった場合はPNG形式になる</param>
        public static void SaveImage(this Canvas canvas, string path, BitmapEncoder encoder = null)
        {
            // レイアウトを再計算させる
            var size = new Size(canvas.ActualWidth, canvas.ActualHeight);
            canvas.Measure(size);
            canvas.Arrange(new Rect(size));

            // VisualObjectをBitmapに変換する
            var renderBitmap = new RenderTargetBitmap((int)size.Width,       // 画像の幅
                                                      (int)size.Height,      // 画像の高さ
                                                      96.0d,                 // 横96.0DPI
                                                      96.0d,                 // 縦96.0DPI
                                                      PixelFormats.Pbgra32); // 32bit(RGBA各8bit)
            renderBitmap.Render(canvas);

            // 出力用の FileStream を作成する
            using (var stream = new System.IO.FileStream(path, System.IO.FileMode.Create))
            {
                // 変換したBitmapをエンコードしてFileStreamに保存する。
                // BitmapEncoder が指定されなかった場合は、PNG形式とする。
                encoder = encoder ?? new PngBitmapEncoder();
                encoder.Frames.Add(BitmapFrame.Create(renderBitmap));
                encoder.Save(stream);
            }
        }

    }

}
